@@ -1,11 +1,11 @@
 use super::Matcher;
 
+use crate::language::Language;
 use crate::meta_var::MetaVarEnv;
 use crate::node::KindId;
-use crate::{Doc, Language, Node};
+use crate::{Doc, Node};
 
 use std::borrow::Cow;
-use std::marker::PhantomData;
 
 use bit_set::BitSet;
 use thiserror::Error;
@@ -23,22 +23,18 @@ pub enum KindMatcherError {
 }
 
 #[derive(Clone)]
-pub struct KindMatcher<L: Language> {
+pub struct KindMatcher {
   kind: KindId,
-  lang: PhantomData<L>,
 }
 
-impl<L: Language> KindMatcher<L> {
-  pub fn new(node_kind: &str, lang: L) -> Self {
+impl KindMatcher {
+  pub fn new<L: Language>(node_kind: &str, lang: L) -> Self {
     Self {
-      kind: lang
-        .get_ts_language()
-        .id_for_node_kind(node_kind, /*named*/ true),
-      lang: PhantomData,
+      kind: lang.kind_to_id(node_kind),
     }
   }
 
-  pub fn try_new(node_kind: &str, lang: L) -> Result<Self, KindMatcherError> {
+  pub fn try_new<L: Language>(node_kind: &str, lang: L) -> Result<Self, KindMatcherError> {
     let s = Self::new(node_kind, lang);
     if s.is_invalid() {
       Err(KindMatcherError::InvalidKindName(node_kind.into()))
@@ -48,10 +44,7 @@ impl<L: Language> KindMatcher<L> {
   }
 
   pub fn from_id(kind: KindId) -> Self {
-    Self {
-      kind,
-      lang: PhantomData,
-    }
+    Self { kind }
   }
 
   /// Whether the kind matcher contains undefined tree-sitter kind.
@@ -81,8 +74,8 @@ pub mod kind_utils {
   }
 }
 
-impl<L: Language> Matcher<L> for KindMatcher<L> {
-  fn match_node_with_env<'tree, D: Doc<Lang = L>>(
+impl Matcher for KindMatcher {
+  fn match_node_with_env<'tree, D: Doc>(
     &self,
     node: Node<'tree, D>,
     _env: &mut Cow<MetaVarEnv<'tree, D>>,
@@ -106,10 +99,10 @@ mod test {
   use super::*;
   use crate::language::Tsx;
   use crate::matcher::MatcherExt;
-  use crate::{Root, StrDoc};
+  use crate::{tree_sitter::StrDoc, Root};
 
   fn pattern_node(s: &str) -> Root<StrDoc<Tsx>> {
-    Root::new(s, Tsx)
+    Root::str(s, Tsx)
   }
   #[test]
   fn test_kind_match() {
@@ -121,7 +114,7 @@ mod test {
       pattern.find_node(cand.clone()).is_some(),
       "goal: {}, candidate: {}",
       kind,
-      cand.to_sexp(),
+      cand.get_inner_node().to_sexp(),
     );
   }
 
@@ -135,7 +128,7 @@ mod test {
       pattern.find_node(cand.clone()).is_none(),
       "goal: {}, candidate: {}",
       kind,
-      cand.to_sexp(),
+      cand.get_inner_node().to_sexp(),
     );
   }
 
