@@ -1,5 +1,6 @@
 mod utils;
 
+use ast_grep_core::NodeMatch;
 use dashmap::DashMap;
 use serde_json::Value;
 use tower_lsp_server::jsonrpc::Result;
@@ -18,7 +19,7 @@ use std::collections::{BTreeMap, HashMap};
 use std::path::PathBuf;
 use std::sync::{Arc, RwLock};
 
-use utils::{convert_match_to_diagnostic, diagnostic_to_code_action, RewriteData};
+use utils::{convert_match_to_diagnostic, diagnostic_to_code_action, Fixes, RewriteData};
 
 pub use tower_lsp_server::{LspService, Server};
 
@@ -31,6 +32,7 @@ struct VersionedAst<D: Doc> {
   version: i32,
   root: AstGrep<D>,
   notes: Notes,
+  fixes: Fixes,
 }
 
 pub struct Backend<L: LSPLang> {
@@ -42,6 +44,8 @@ pub struct Backend<L: LSPLang> {
   interner: DashMap<String, Arc<String>>,
   // rule finding closure to reload rules
   rule_finder: Box<dyn Fn() -> anyhow::Result<RuleCollection<L>> + Send + Sync>,
+  // store client capabilities to check support
+  capabilities: Arc<RwLock<ClientCapabilities>>,
 }
 
 const FALLBACK_CODE_ACTION_PROVIDER: Option<CodeActionProviderCapability> =
@@ -76,6 +80,10 @@ fn code_action_provider(
 
 impl<L: LSPLang> LanguageServer for Backend<L> {
   async fn initialize(&self, params: InitializeParams) -> Result<InitializeResult> {
+    let code_action_provider = code_action_provider(&params.capabilities);
+    if let Ok(mut cap) = self.capabilities.write() {
+      *cap = params.capabilities;
+    }
     Ok(InitializeResult {
       server_info: Some(ServerInfo {
         name: "ast-grep language server".to_string(),
@@ -84,8 +92,7 @@ impl<L: LSPLang> LanguageServer for Backend<L> {
       capabilities: ServerCapabilities {
         // TODO: change this to incremental
         text_document_sync: Some(TextDocumentSyncCapability::Kind(TextDocumentSyncKind::FULL)),
-        code_action_provider: code_action_provider(&params.capabilities)
-          .or(FALLBACK_CODE_ACTION_PROVIDER),
+        code_action_provider: code_action_provider.or(FALLBACK_CODE_ACTION_PROVIDER),
         execute_command_provider: Some(ExecuteCommandOptions {
           commands: vec![APPLY_ALL_FIXES.to_string()],
           work_done_progress_options: Default::default(),
@@ -226,6 +233,7 @@ impl<L: LSPLang> Backend<L> {
       map: DashMap::new(),
       interner: DashMap::new(),
       rule_finder: Box::new(rule_finder),
+      capabilities: Arc::new(RwLock::new(ClientCapabilities::default())),
     }
   }
 
@@ -269,7 +277,7 @@ impl<L: LSPLang> Backend<L> {
     &self,
     uri: &Uri,
     versioned: &VersionedAst<StrDoc<L>>,
-  ) -> Option<Vec<Diagnostic>> {
+  ) -> Option<(Vec<Diagnostic>, Fixes)> {
     let path = self.uri_to_relative_path(uri)?;
 
     let rules = self.rules.read().ok()?;
@@ -283,11 +291,19 @@ impl<L: LSPLang> Backend<L> {
     scan.set_unused_suppression_rule(&unused_suppression_rule);
     let matches = scan.scan(&versioned.root, false).matches;
     let mut diagnostics = vec![];
+    let mut fixes = Fixes::new();
     for (rule, ms) in matches {
-      let to_diagnostic = |m| convert_match_to_diagnostic(uri, m, rule);
+      let to_diagnostic = |m: NodeMatch<StrDoc<L>>| {
+        let diagnostic = convert_match_to_diagnostic(uri, &m, rule);
+        let rewrite_data = RewriteData::from_node_match(&m, rule);
+        if let Some(r) = rewrite_data {
+          fixes.insert((diagnostic.range, rule.id.clone()), r);
+        }
+        diagnostic
+      };
       diagnostics.extend(ms.into_iter().map(to_diagnostic));
     }
-    Some(diagnostics)
+    Some((diagnostics, fixes))
   }
 
   fn build_notes(&self, diagnostics: &[Diagnostic]) -> Notes {
@@ -319,8 +335,10 @@ impl<L: LSPLang> Backend<L> {
     uri: Uri,
     versioned: &mut VersionedAst<StrDoc<L>>,
   ) -> Option<()> {
-    let diagnostics = self.get_diagnostics(&uri, versioned).unwrap_or_default();
+    let (diagnostics, fixes) = self.get_diagnostics(&uri, versioned).unwrap_or_default();
     versioned.notes = self.build_notes(&diagnostics);
+    versioned.fixes = fixes;
+
     self
       .client
       .publish_diagnostics(uri, diagnostics, Some(versioned.version))
@@ -329,6 +347,18 @@ impl<L: LSPLang> Backend<L> {
   }
 
   async fn get_path_of_first_workspace(&self) -> Option<std::path::PathBuf> {
+    // need drop the lock before await
+    let client_support_workspace = {
+      let cap = self.capabilities.read().ok()?;
+      cap
+        .workspace
+        .as_ref()
+        .and_then(|w| w.workspace_folders)
+        .unwrap_or(false)
+    };
+    if !client_support_workspace {
+      return None;
+    }
     let folders = self.client.workspace_folders().await.ok()??;
     let folder = folders.first()?;
     folder.uri.to_file_path().map(PathBuf::from)
@@ -336,7 +366,11 @@ impl<L: LSPLang> Backend<L> {
 
   // skip files outside of workspace root #1382, #1402
   async fn should_skip_file_outside_workspace(&self, text_doc: &TextDocumentItem) -> Option<()> {
-    let workspace_root = self.get_path_of_first_workspace().await?;
+    // fallback to base if no workspace provided by client #2211
+    let workspace_root = self
+      .get_path_of_first_workspace()
+      .await
+      .unwrap_or_else(|| self.base.clone());
     let doc_file_path = text_doc.uri.to_file_path()?;
     if doc_file_path.starts_with(workspace_root) {
       None
@@ -366,6 +400,7 @@ impl<L: LSPLang> Backend<L> {
       version: text_doc.version,
       root,
       notes: BTreeMap::new(),
+      fixes: Fixes::new(),
     };
     self
       .client
@@ -379,7 +414,8 @@ impl<L: LSPLang> Backend<L> {
   async fn on_change(&self, params: DidChangeTextDocumentParams) -> Option<()> {
     let text_doc = params.text_document;
     let uri = text_doc.uri.as_str();
-    let text = &params.content_changes[0].text;
+    let change = &params.content_changes.first()?;
+    let text = &change.text;
     self
       .client
       .log_message(MessageType::LOG, "Parsing changed doc.")
@@ -395,6 +431,7 @@ impl<L: LSPLang> Backend<L> {
       version: text_doc.version,
       root,
       notes: BTreeMap::new(),
+      fixes: Fixes::new(),
     };
     self
       .client
@@ -421,24 +458,31 @@ impl<L: LSPLang> Backend<L> {
       .map
       .get(uri.as_str())
       .ok_or(LspError::UnsupportedFileType)?;
-    let mut diagnostics = self
+    let (_diagnostics, fixes) = self
       .get_diagnostics(&uri, &versioned)
       .ok_or(LspError::NoActionableFix)?;
-    diagnostics.sort_by_key(|d| (d.range.start, d.range.end));
+
+    let mut entries: Vec<_> = fixes.iter().collect();
+    entries.sort_by(|((range_a, _), _), ((range_b, _), _)| {
+      range_a
+        .start
+        .cmp(&range_b.start)
+        .then(range_a.end.cmp(&range_b.end))
+    });
+
     let mut last = Position {
       line: 0,
       character: 0,
     };
-    let edits: Vec<_> = diagnostics
+    let edits: Vec<TextEdit> = entries
       .into_iter()
-      .filter_map(|d| {
-        if d.range.start < last {
+      .filter_map(|((range, _id), rewrite_data)| {
+        if range.start < last {
           return None;
         }
-        let rewrite_data = RewriteData::from_value(d.data?)?;
         let fixed = rewrite_data.fixers.first()?.fixed.to_string();
-        let edit = TextEdit::new(d.range, fixed);
-        last = d.range.end;
+        let edit = TextEdit::new(*range, fixed);
+        last = range.end;
         Some(edit)
       })
       .collect();
@@ -483,6 +527,10 @@ impl<L: LSPLang> Backend<L> {
       return None;
     }
     let text_doc = params.text_document;
+
+    let document = self.map.get(text_doc.uri.as_str())?;
+    let fixes_cache = &document.fixes;
+
     let response = params
       .context
       .diagnostics
@@ -493,7 +541,7 @@ impl<L: LSPLang> Backend<L> {
           .map(|s| s.contains("ast-grep"))
           .unwrap_or(false)
       })
-      .filter_map(|d| diagnostic_to_code_action(&text_doc, d))
+      .filter_map(|d| diagnostic_to_code_action(&text_doc, d, fixes_cache))
       .flatten()
       .map(CodeActionOrCommand::from)
       .collect();
@@ -671,10 +719,12 @@ impl<L: LSPLang> Backend<L> {
         continue;
       };
       // Republish diagnostics for this file
-      let Some(diagnostics) = self.get_diagnostics(&uri, versioned) else {
-        continue;
+      let (diagnostics, fixes) = match self.get_diagnostics(&uri, versioned) {
+        Some((d, f)) => (d, f),
+        None => (Vec::new(), HashMap::new()),
       };
       versioned.notes = self.build_notes(&diagnostics);
+      versioned.fixes = fixes;
       self
         .client
         .publish_diagnostics(uri, diagnostics, Some(versioned.version))
